package com.dayorganiser.jgantt.drawing.plotter;

import com.dayorganiser.jgantt.drawing.Plotter;
import com.dayorganiser.jgantt.drawing.Proxy;

import com.dayorganiser.jgantt.drawing.scheme.Scheme;

import com.dayorganiser.jgantt.drawing.scheme.color.GradientScheme;

import com.dayorganiser.jgantt.drawing.scheme.range.SchemeRange;

import com.dayorganiser.util.Trace;

import java.awt.Color;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import java.util.Map;

public abstract class AbstractPlotter implements Plotter
{
	private static final String THIS = AbstractPlotter.class.getName();
	private static final boolean TRACE = Trace.getEnable(THIS);

	//__________________________________________________________________________

	protected boolean setFont(Object key, Graphics g, String schemeKey, Proxy p, Map attribs)
	{
		Scheme scheme = getScheme(schemeKey, p, p.getAttributes(key));
		if (scheme == null)
		   return false;

		SchemeRange range = p.getRange(key, schemeKey);
		int index = range.indexOf(key);

	   g.setFont( (Font) scheme.getObjectAt(index, null));
		return true;
	}

	//__________________________________________________________________________

	protected boolean setScheme(Object key, Graphics g, String schemeKey,
		Proxy p, Map attribs)
	{
		Rectangle r = g.getClipBounds();
		return setScheme(key, g,  r.x, r.y, r.width, r.height, schemeKey, p, attribs);
	}

	protected boolean setScheme(Object key, Graphics g,
		int x1, int y1, int x2, int y2, String schemeKey, Proxy p, Map attribs)
	{
		Scheme scheme = getScheme(schemeKey, p,  p.getAttributes(key));
		if (scheme == null)
		   return false;

		SchemeRange range = p.getRange(key, schemeKey);
		int index = range.indexOf(key);

		Graphics2D g2d = (Graphics2D) g;

		if (scheme instanceof GradientScheme)
		{
		   // note a Gradient paint hols the second colours as well
		   // to calc the index take the range and add the index i.e. offset
			g2d.setPaint(new GradientPaint(x1, y2, (Color) scheme.getObjectAt(index, range),
				x2, y2, (Color) scheme.getObjectAt(index + range.getFinish() - range.getStart(),
				range), ((GradientScheme) scheme).isCyclic()));
			return true;
		}

	   g.setColor( (Color) scheme.getObjectAt(index, null));
		return true;
	}

	//___________________________________________________________________________

	private Scheme getScheme(String schemeKey, Proxy proxy, Map attribs)
	{
		Object schemeID = attribs.get(schemeKey);
		if (schemeID == null)
		{
			Trace.out.println(THIS + ".paintEntry() missing schemeID for " + schemeKey);
		   return null;
		}
		return proxy.getScheme(schemeID);
	}
}