package com.dayorganiser.jgantt.drawing.plotter.axis;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;

import java.text.SimpleDateFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.dayorganiser.jgantt.model.Axis;

import com.dayorganiser.jgantt.drawing.Proxy;

import com.dayorganiser.jgantt.drawing.plotter.AbstractPlotter;

import com.dayorganiser.jgantt.model.axis.TimelineConstants;

public class TimelinePlotter extends AbstractPlotter
{
	/** a little assumption that no-one will ever care to change the first day of
		week. Whilst the timeline is open, if ths is a requirement for your project
		feel free to edit the source/let us know this is a requirement*/
	protected final int FIRST_DAY_IN_WEEK = Calendar.getInstance().getFirstDayOfWeek();

	/**
		Data types they are numbered by granularity/priority one colour type will
		override the other based on the priority */
	public int YEAR = -1; /** use YEAR for all years or YEAR - currentYear we'll never support BC*/
	public int DEFAULT = 0;
	public int DAY = 1; /** DAY indicates all days or DAY +22 for the 22nd*/

	protected int DOW = 39; /** use only to calculate dow notice internal use only*/
	public int MONDAY =  40;
	public int TUESDAY = 41;
	public int WEDENSDAY = 42;
	public int THURSDAY = 43;
	public int FRIDAY = 44;
	public int SATURDAY = 45;
	public int SUNDAY = 46;
	public int WEEKEND = 47; /* if you want sat/sun to be different colours don't use this **/
	public int HOURS = 48; /** HOURS or HOURS +11*/
	public int MINUTES = 100; /** MINUTES +43*/

	// Default styles types
	public int LINE_COLOR = 0;
	public int FONT_COLOR = 1;
	public int FONT = 2;
	public int FG_COLOR = 3;
	public int BG_COLOR = 4;

	public boolean smartRendering = true;  // smart rendering will disallow rendering things
														// that don't really fit well

	protected boolean paintTop = false;
	protected Axis axis;
	protected Map drawingOptions = new HashMap(); // String SchemeName --> ColourScheme
	{
		drawingOptions.put(DEFAULT + ":" + BG_COLOR ,Color.white);
		drawingOptions.put(DEFAULT + ":" + FG_COLOR ,Color.white);
		drawingOptions.put(DEFAULT + ":" + FONT_COLOR ,Color.black);
		drawingOptions.put(DEFAULT + ":" + LINE_COLOR, Color.black);
		drawingOptions.put(DEFAULT + ":" + FONT,
		   new Font("Helvetica",Font.PLAIN,10));
	}

	/** no real effort has been made to seperate the DefaultTimelinePanel
		from the of the code.. if many people are really wanting this then
		it can be done. */
	protected Map formatString = new HashMap();
	{
		String years[] = {"yyyy"};
		formatString.put(new Integer(TimelineConstants.GRANULARITY_YEAR),years);

		String days [] = {"dd-MMM-yy","dd-MMM", "dd"};
		formatString.put(new Integer(TimelineConstants.GRANULARITY_DAY), days);

		String hours [] = {"EEE dd-MMM-yy hh:00","hh"};
		formatString.put(new Integer(TimelineConstants.GRANULARITY_HOUR),hours);

		String minutes[] ={"EEE dd-MMM-yy hh:mm","mm"};
		formatString.put(new Integer(TimelineConstants.GRANULARITY_MINUTE),minutes);

		String months[] = {"MMM yyyy","MMM"};
		formatString.put(new Integer(TimelineConstants.GRANULARITY_MONTH),months);

		String seconds[] = {"ss:mm","ss"};
		formatString.put(new Integer(TimelineConstants.GRANULARITY_SECOND),seconds);
	}

	public TimelinePlotter(Axis axis, boolean paintTop)
	{
		this.axis = axis;
		this.paintTop = paintTop;
	}

	//___________________________________________________________________________

	protected Object getDrawingOption(int data, int type, Object def)
	{
		Object o = drawingOptions.get(data + ":" + type);
		return o != null ? o : def;
	}

	/** returns the appropriate action fot the type'd granularity*/
	protected Object getGranularityOption(long timeValue, int granularity, int type)
	{
		// default type should never return null
		Object o = getDrawingOption(DEFAULT,type, null);
		if (timeValue <= 0)
		   return o;

		Calendar c = Calendar.getInstance();
		c.setTime(new Date(timeValue));

		switch (granularity)
		{
		   case TimelineConstants.GRANULARITY_DAY:
		   {
				// returns the current week day
				int weekDay = c.get(Calendar.DAY_OF_WEEK);
				o = getDrawingOption(WEEKEND, type, o);

			   // equivalant of draw weekends
				if (weekDay == Calendar.SUNDAY || weekDay == Calendar.SATURDAY)
				o =  getDrawingOption(DOW + weekDay, type, o);
				break;
		   }
		}
		return o;
	}

	//___________________________________________________________________________

	public boolean highlight(long value)
	{
		return true;
	}

	//___________________________________________________________________________

	public Rectangle paintComponent(Object o, Graphics g, int visX, int visY, Proxy proxy, Map map)
	{
		int width = g.getClipBounds().width;

		int lastGranularity = Integer.MIN_VALUE;
		while (true)
		{
		   int strWidth = lastGranularity == Integer.MIN_VALUE ?
				20 : g.getFontMetrics().stringWidth(
				formatDate(new Date( ((Long)axis.getValue(width, 0)).longValue()),lastGranularity));

		   int granularity = axis.getGranularity(width,strWidth);
		   if (lastGranularity >= granularity)
				break;

		   lastGranularity = granularity;
		}

		switch (lastGranularity)
		{
		   case TimelineConstants.GRANULARITY_YEAR:
				paintGranularity(TimelineConstants.GRANULARITY_YEAR, TimelineConstants.GRANULARITY_MONTH, g);
				break;

		   case TimelineConstants.GRANULARITY_MONTH:
				paintGranularity(TimelineConstants.GRANULARITY_MONTH, TimelineConstants.GRANULARITY_DAY, g);
				break;

		   case TimelineConstants.GRANULARITY_DAY:
				paintGranularity(TimelineConstants.GRANULARITY_DAY, TimelineConstants.GRANULARITY_HOUR, g);
				break;

		   case TimelineConstants.GRANULARITY_HOUR:
				paintGranularity(TimelineConstants.GRANULARITY_HOUR, TimelineConstants.GRANULARITY_MINUTE, g);
				break;

		   case TimelineConstants.GRANULARITY_MINUTE:
				paintGranularity(TimelineConstants.GRANULARITY_MINUTE, TimelineConstants.GRANULARITY_SECOND, g);
  				break;

		   case TimelineConstants.GRANULARITY_SECOND:
				paintGranularity(TimelineConstants.GRANULARITY_MINUTE, TimelineConstants.GRANULARITY_SECOND, g);
				break;
		}
		return null;
	}

	//___________________________________________________________________________

	/** this is a helper method */
	protected void paintGranularity(int top, int bottom, Graphics g)
	{
		int gran = paintTop ? top : bottom;
		paintGranularity(g, gran, g.getClipBounds().width, false);
	}

	//___________________________________________________________________________

	/** this should be fed with a graphics which is the unprintable portion of the screen*/
	protected void paintHeader(Graphics g,int granularity)
	{
		g.setColor( (Color) getGranularityOption(-1,
		   granularity, FG_COLOR));

		g.drawLine(0,0,g.getClipBounds().width,0); //top line
		//we need to do more than this
	}

	protected void paintSplitter(Graphics g, int granularity)
	{
		g.setColor( (Color) getGranularityOption(-1,
			TimelineConstants.GRANULARITY_DAY, LINE_COLOR));

		int h = g.getClipBounds().height;
		int width = g.getClipBounds().width;
		int y = h/2;
		g.drawLine(0,y,width,y); // seperator
	}

	protected void paintGranularity(Graphics g, int granularity, int timeWidth, boolean top)
	{
		int units = (int) axis.getWidth(timeWidth,granularity);
		if (units < 1) //not drawing anything a pixel or less
		   return;

	   long lastTimeValue = 0;
		int lastWidth = 0;
		int width = g.getClipBounds().width;
		int height = g.getClipBounds().height;

		boolean firstLoop = true;
		boolean renderText = true;

		// an array of string containing the date rendering attribs
		String values []  = (String []) formatString.get(new Integer(granularity));
		SimpleDateFormat defaultFormat = new SimpleDateFormat(values[values.length -1]);

		int textSpacer = top ? 10 : 2;

		// not a safe idea.. to draw things we can't
		if (textSpacer > width)
		   return;

		for (Iterator iter = axis.visibleIterator(granularity); iter.hasNext();)
		{
		   long timeValue = ((Long)iter.next()).longValue();

		   // this is mean't to calculate the point at which the date starts
		   int i = Math.max(axis.getPoint(timeWidth,new Long(timeValue)),0);

			// calculate offset
			int offset = i - lastWidth;

			if (!firstLoop && offset <= 3)
				continue;

	      // font must be set here for line calcs to work
	   	g.setFont( (Font) getGranularityOption(lastTimeValue, granularity,FONT));

			if (!firstLoop)
			{
				g.setColor( (Color) getGranularityOption(lastTimeValue,
				   granularity, LINE_COLOR));

				int y = height / 2;
	  		   int ascent = new Double(g.getFontMetrics().getAscent() / 1.5).intValue();

				if (top)
				{
				   g.drawLine(i, y - ascent , i, height);
				   g.drawLine(i+7, 1, i, y -ascent);
				}
				else
				   g.drawLine(i,0,i,5);

				g.setColor( (Color) getGranularityOption(timeValue,
					granularity, FG_COLOR));

				if (top)
					g.drawLine(i+1, y - ascent , i+1, height);
				else
					g.drawLine(i+1,0,i+1,5);
			}
			else
				renderText = g.getFontMetrics().
					getStringBounds(defaultFormat.format(new Date(lastTimeValue)),g).getBounds().width + textSpacer	< units;

			if (renderText)
			{
        		g.setColor( (Color) getGranularityOption(timeValue, granularity, FONT_COLOR));
				paintDate(g.create(Math.max(i-units,0) + textSpacer,0,
				Math.max(0,offset),height),lastTimeValue, -1,values);
			}

			//save the last width
			firstLoop = false;
			lastWidth = i;
			lastTimeValue = timeValue;
		}

		if (renderText)
			paintDate(g.create(lastWidth + textSpacer,0,width -textSpacer,height),lastTimeValue, -1,values);
	}

	/** returns the level */
	protected int paintDate(Graphics g, long timeValue, int level, String values [])
	{
		int width = g.getClipBounds().width;
		Date date = new Date(timeValue);	//cached
  		for (level=0; level < values.length; level++)
  		{
		   SimpleDateFormat format = new SimpleDateFormat(values[level]);
			String value  = format.format(date);

			if (g.getFontMetrics().getStringBounds(value,g).getBounds().width + 10 > width)
				continue;

			g.drawString(value, 0,g.getFontMetrics().getHeight() / 2  +3);
			return level;
  		}
		return -1;
	}

	//___________________________________________________________________________

	protected String formatDate(Date date, int granularity)
	{
		String values []  = (String []) formatString.get(new Integer(granularity));
		SimpleDateFormat format = new SimpleDateFormat(values[0]);
		return format.format(date);
	}
}