package com.dayorganiser.jgantt.drawing.proxy.factory;

import com.dayorganiser.jgantt.drawing.Proxy;

import com.dayorganiser.util.text.StringUtils;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import java.net.URL;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
  * will instatiate immedietly or on demand a set of proxies from your config directory
 **/
public class DefaultProxyFactory implements ProxyFactory
{
	protected Map loadedClasses; // the classes that are loaded
	protected Map toLoad; // the classes left to load

	public DefaultProxyFactory(URL url)
	{
		this(url, false);
	}

	public DefaultProxyFactory(URL url, boolean instantiate)
	{
		// load up the proxy config from disk
		this.loadedClasses = loadURL(url);
		this.toLoad = new HashMap();

		// instantiate all of the classes left on the disk
		if (instantiate)
		   for (Iterator iter = toLoad.keySet().iterator(); iter.hasNext();)
				instantiateProxy((String) iter.next());
	}

	//___________________________________________________________________________

	public Proxy getProxy(String name)
	{
		Proxy proxy = (Proxy) loadedClasses.get(name);
		return proxy != null ? proxy : instantiateProxy(name);
	}

	//___________________________________________________________________________

	protected Proxy instantiateProxy(String name)
	{
		Map properties = (Map) toLoad.get(name);
		if (properties == null)
		   return null;
		return null;
	}

	//___________________________________________________________________________

	protected Map loadURL(URL url)
	{
		HashMap results = new HashMap();
		InputStreamReader in = null;
		try
		{
			in = new InputStreamReader(url.openStream());
			BufferedReader reader = new BufferedReader(in);

		   if (reader.ready())
			   for (String value = reader.readLine(); reader.ready();
				   value = reader.readLine())
				{
					String values [] = StringUtils.splitAt(value,'=');
				   if (values.length < 3)
						continue;

				   String keys [] = StringUtils.splitAt(values[0],'_');
				   String domain = values[1];
				   String attribs = values[2];

				   // lets marge them with the current set
				   createMap(results, keys, values, 0);

				}
	   	reader.close();
		}
		catch (IOException io)
		{
		}
		return results;
	}

	//___________________________________________________________________________

	private void createMap(Map pos, String keys[], String values[], int i)
	{
		if (keys.length == i)
		{
		   pos.put(keys[i-1], values);
		   return;
		}

		Map newPos = (Map) pos.get(keys[i]);
		if (newPos == null)
		{
			newPos = new HashMap();
   	   pos.put(keys[i], newPos);
		}

		createMap(newPos, keys, values, ++i);
	}
}