package com.dayorganiser.jgantt.drawing.scheme.color;

import com.dayorganiser.jgantt.drawing.scheme.EnumeratedScheme;
import com.dayorganiser.jgantt.drawing.scheme.Scheme;
import com.dayorganiser.jgantt.drawing.scheme.SchemeProvider;

import com.dayorganiser.jgantt.drawing.scheme.color.GradientScheme;
import com.dayorganiser.jgantt.drawing.scheme.color.HSBScheme;

import com.dayorganiser.util.text.StringUtils;

import java.awt.Color;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import java.net.URL;

import java.util.HashSet;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class ColorSchemeProvider implements SchemeProvider
{
	public final static String ENUM_SCHEME = "ENUM";
	public final static String HSB_SCHEME = "HSB";
	public final static String GRADIENT_SCHEME = "GRADIENT";

	protected Map schemes;

	public ColorSchemeProvider(URL url)
	{
		this.schemes = load(url);
	}

	//___________________________________________________________________________

	public Scheme getScheme(String name)
	{
		return (Scheme) schemes.get(name);
	}

	//___________________________________________________________________________

	protected Map load(URL url)
	{
		HashMap results = new HashMap();
		HashMap types = new HashMap();

		InputStreamReader in = null;
		try
		{
			in = new InputStreamReader(url.openStream());
			BufferedReader reader = new BufferedReader(in);

		   boolean ready = reader.ready();
		   if (reader.ready())
			   for (String value = reader.readLine(); ready;
				   value = reader.readLine())
				{
				   ready = reader.ready();
					String values [] = StringUtils.splitAt(value,'=');
				   if (values.length != 2)
						continue;

				   String key = values[0];
				   values = StringUtils.splitAt(values[1],',');

				   types.put(key, values[0]);

				   Set colors = (Set) results.get(key);
				   if (colors == null)
				   {
						colors = new HashSet();
						results.put(key, colors);
				   }

   				Color color = new Color(Integer.parseInt(values[1]),
  				      Integer.parseInt(values[2]),
		  				Integer.parseInt(values[3]));

					colors.add(color);
				}
	   	reader.close();
		}
		catch (IOException io)
		{
		}

		// convert the results
		HashMap finalResults = new HashMap();
		for (Iterator iter = types.entrySet().iterator(); iter.hasNext();)
		{
		   Map.Entry entry = (Map.Entry) iter.next();

		   String key = (String) entry.getKey();
		   String type = (String) entry.getValue();

		   Set colors = (Set) results.get(key);
		   Scheme scheme = null;
			if (type.equals(ENUM_SCHEME))
				scheme = new EnumeratedScheme(colors);
		   else  if (type.equals(GRADIENT_SCHEME))
				scheme = new GradientScheme(colors);
		   else if (type.equals(HSB_SCHEME))
				scheme = new HSBScheme(colors);
		   if (scheme != null)
				finalResults.put(key, scheme);
		}
		return finalResults;
	}
}
