package com.dayorganiser.jgantt.drawing.scheme.font;

import com.dayorganiser.jgantt.drawing.scheme.EnumeratedScheme;
import com.dayorganiser.jgantt.drawing.scheme.Scheme;
import com.dayorganiser.jgantt.drawing.scheme.SchemeProvider;

import com.dayorganiser.util.text.StringUtils;

import java.awt.Font;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import java.net.URL;

import java.awt.font.TextAttribute;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class FontProvider implements SchemeProvider
{
	protected static Map fontKeys = new HashMap();
	static
	{
		fontKeys.put("background", TextAttribute.BACKGROUND);

		fontKeys.put("bidi_embedding", TextAttribute.BIDI_EMBEDDING);

		fontKeys.put("char_replacement", TextAttribute.CHAR_REPLACEMENT);

		fontKeys.put("font", TextAttribute.FONT);

		fontKeys.put("family", TextAttribute.FAMILY);

		fontKeys.put("foreground", TextAttribute.FOREGROUND);

		fontKeys.put("input_method_highlight", TextAttribute.INPUT_METHOD_HIGHLIGHT);
		fontKeys.put("input_method_underlind", TextAttribute.INPUT_METHOD_UNDERLINE);

		fontKeys.put("justification", TextAttribute.JUSTIFICATION);
		fontKeys.put("justification_full", TextAttribute.JUSTIFICATION_FULL);
		fontKeys.put("justification_none", TextAttribute.JUSTIFICATION_NONE);

		fontKeys.put("posture", TextAttribute.POSTURE);
		fontKeys.put("posture_oblique", TextAttribute.POSTURE_OBLIQUE);
		fontKeys.put("posture_regular", TextAttribute.POSTURE_REGULAR);

		fontKeys.put("run_direction", TextAttribute.RUN_DIRECTION);
		fontKeys.put("run_direction_ltr", TextAttribute.RUN_DIRECTION_LTR);
		fontKeys.put("run_direction_rtl", TextAttribute.RUN_DIRECTION_RTL);

		fontKeys.put("size", TextAttribute.SIZE);

		fontKeys.put("strikethrough", TextAttribute.STRIKETHROUGH);
		fontKeys.put("strikethrough_on", TextAttribute.STRIKETHROUGH_ON);

		fontKeys.put("superscript", TextAttribute.SUPERSCRIPT);
		fontKeys.put("superscript_sub", TextAttribute.SUPERSCRIPT_SUB);
		fontKeys.put("superscript_super", TextAttribute.SUPERSCRIPT_SUPER);

		fontKeys.put("swap_colors", TextAttribute.SWAP_COLORS);
		fontKeys.put("swap_colors_on", TextAttribute.SWAP_COLORS_ON);

		fontKeys.put("transform", TextAttribute.TRANSFORM);

		fontKeys.put("underline", TextAttribute.UNDERLINE);
		fontKeys.put("underline_low_dashed", TextAttribute.UNDERLINE_LOW_DASHED);
		fontKeys.put("underline_low_grey", TextAttribute.UNDERLINE_LOW_GRAY);
		fontKeys.put("underline_low_one_pixel", TextAttribute.UNDERLINE_LOW_ONE_PIXEL);
		fontKeys.put("underline_low_two_pixel", TextAttribute.UNDERLINE_LOW_TWO_PIXEL);
		fontKeys.put("underline_on", TextAttribute.UNDERLINE_ON);

		fontKeys.put("weight", TextAttribute.WEIGHT);
		fontKeys.put("weight_bold", TextAttribute.WEIGHT_BOLD);
		fontKeys.put("weight_demibold", TextAttribute.WEIGHT_DEMIBOLD);
		fontKeys.put("weight_demilight", TextAttribute.WEIGHT_DEMILIGHT);
		fontKeys.put("weight_extrabold", TextAttribute.WEIGHT_EXTRABOLD);
		fontKeys.put("weight_extra_light", TextAttribute.WEIGHT_EXTRA_LIGHT);
		fontKeys.put("weight_heavy", TextAttribute.WEIGHT_HEAVY);
		fontKeys.put("weight_light", TextAttribute.WEIGHT_LIGHT);
		fontKeys.put("weight_medium", TextAttribute.WEIGHT_MEDIUM);
		fontKeys.put("weight_regular", TextAttribute.WEIGHT_REGULAR);
		fontKeys.put("weight_semibold", TextAttribute.WEIGHT_SEMIBOLD);
		fontKeys.put("weight_ultrabold", TextAttribute.WEIGHT_ULTRABOLD);

		fontKeys.put("width", TextAttribute.WIDTH);
		fontKeys.put("width_condensed", TextAttribute.WIDTH_CONDENSED);
		fontKeys.put("width_extended", TextAttribute.WIDTH_EXTENDED);
		fontKeys.put("width_regular", TextAttribute.WIDTH_REGULAR);
		fontKeys.put("width_semi_condensed", TextAttribute.WIDTH_SEMI_CONDENSED);
		fontKeys.put("width_semi_extended", TextAttribute.WIDTH_SEMI_EXTENDED);
	}

	protected Map schemes;

	public FontProvider(URL url)
	{
		this.schemes = load(url);
	}

	//___________________________________________________________________________

	public Scheme getScheme(String name)
	{
		return (Scheme) schemes.get(name);
	}

	//___________________________________________________________________________

	protected Map load(URL url)
	{
		HashMap results = new HashMap();

		InputStreamReader in = null;
		try
		{
			in = new InputStreamReader(url.openStream());
			BufferedReader reader = new BufferedReader(in);

		   boolean ready = reader.ready();
		   if (reader.ready())
			   for (String value = reader.readLine(); ready;
				   value = reader.readLine())
				{
				   ready = reader.ready();
					String values [] = StringUtils.splitAt(value,'=');
				   if (values.length != 2)
						continue;

				   String key = values[0];
				   values = StringUtils.splitAt(values[1],',');

				   Map attributes = new HashMap();
				   for (int i=1; i < values.length; i+=2)
				   {
						Object k = fontKeys.get(values[i -1] );
						if (k == null)
						   k = values[i -1];

						Object v = fontKeys.get(values [i]);
						if ( v== null)
							v = values[i];

						// convert where possible to the correct type
						try
						{
							if (v instanceof String)
						   	v = new Float( (String) v );
						}
						catch (NumberFormatException ne)
						{
						}
					   attributes.put(k, v);
				   }

				   Set collection = (Set) results.get(key);
					if (collection == null)
				   {
					   collection = new HashSet();
						results.put(key, collection);
				   }
				   collection.add(Font.getFont(attributes));
				}
	   	reader.close();
		}
		catch (IOException io)
		{
		}

		// convert the results
		HashMap finalResults = new HashMap();
		for (Iterator iter = results.entrySet().iterator(); iter.hasNext();)
		{
		   Map.Entry entry = (Map.Entry) iter.next();

		   String key = (String) entry.getKey();

			Scheme scheme = new EnumeratedScheme((Set) entry.getValue());
			finalResults.put(key, scheme);
		}
		return finalResults;
	}
}
