package com.dayorganiser.jgantt.model;

import com.dayorganiser.jgantt.model.axis.AxisListener;

import com.dayorganiser.util.Interval;

import java.util.Iterator;

public interface Axis
{
	public String getName();

	//___________________________________________________________________________

	/** +values will zoom in x millis the reverse applies for -values*/
	boolean zoom(int step);

	//___________________________________________________________________________

	/** returns the total interval of time we are using*/
	Interval getInterval();

	/** returns the visible interval */
	Interval getVisible();

	//___________________________________________________________________________

	void setInterval(Interval interval);

	/** sets the visible interval will return false, if the visible interval exists
		outside of the interval */
	boolean setVisible(Interval visible);

	//___________________________________________________________________________

	/** estimates the correct granularity */
	int getGranularity(int width, int minWidth);

	/** returns the value at the correct location */
	Object getValue(int width, int point);

	/** returns the location for the value, virtual if not visible*/
	int getPoint(int width, Object value);

	/** you should only call this for elements lower than the current granularity
		however it should work in all cases */
	double getWidth(int width, int granularity);

	//___________________________________________________________________________

	/** is my location visible on this axis ?*/
	boolean isVisible(Object value);

	/** returns an interator that will return all times that are contained with in
		this time period */
	Iterator visibleIterator(int granularity);

	//_Listeners_________________________________________________________________

	/** adds a new timeline listener*/
	void addAxisListener(AxisListener listener);

	/** removes a timeline listener*/
	void removeAxisListener(AxisListener listener);
}