package com.dayorganiser.jgantt.model.axis;

import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import com.dayorganiser.jgantt.model.Axis;

import com.dayorganiser.jgantt.model.axis.AxisListener;

import com.dayorganiser.util.time.DefaultTimeInterval;
import com.dayorganiser.util.time.TimeInterval;

import com.dayorganiser.util.Interval;

public abstract class AbstractAxis implements Axis
{
	protected Interval interval;
	protected Interval visible;
	protected String name;

	protected Vector listeners = new Vector(2);

	public AbstractAxis(String name)
	{
		this.name = name;
	}

	public String getName()
	{
		return name;
	}

	//___________________________________________________________________________

	public Interval getInterval()
	{
		return interval;
	}

	public Interval getVisible()
	{
		return visible;
	}

	//___________________________________________________________________________

	public synchronized boolean setVisible(Interval interval)
	{
	   if (! this.interval.contains(interval))
		   return false;

   	this.visible = interval; // make changes

   	fireZoomChanged(); // notify changes
		return true;
	}

	//___________________________________________________________________________

	public boolean contains(Interval interval)
	{
		return this.interval.contains(interval);
	}

	public boolean intersects(Interval interval)
	{
		return this.interval.intersects(interval);
	}

	//___________________________________________________________________________

	/** isVisible should be noticably faster than the equivelant getPoint which requires a calculation*/
	public boolean isVisible(Object value)
	{
		return this.interval.containsValue(value); //we don't know so it's true
	}

	public boolean isVisible(Interval interval)
	{
		return this.visible.intersects(interval);
	}

	//___________________________________________________________________________

	public synchronized void setInterval(Interval interval)
	{
		this.interval = interval;  // this is not safe
		this.visible = interval;
	}

	//_Listeners_________________________________________________________________

	/** adds a new timeline listener*/
	public void addAxisListener(AxisListener listener)
	{
		synchronized (listeners)
		{
		   listeners.add(listener);
		}
	}

	/** removes a timeline listener*/
	public void removeAxisListener(AxisListener listener)
	{
		synchronized (listeners)
		{
		   listeners.remove(listeners);
		}
	}

	//___________________________________________________________________________

	protected void fireZoomChanged()
	{
		synchronized (listeners)
		{
		   for (Iterator iter = listeners.iterator(); iter.hasNext();)
			   ((AxisListener) iter.next()).granularityChanged();
		}
	}
}