package com.dayorganiser.jgantt.model.axis;

import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import com.dayorganiser.jgantt.model.Axis;

import com.dayorganiser.jgantt.model.axis.AbstractAxis;

import com.dayorganiser.util.time.DefaultTimeInterval;
import com.dayorganiser.util.time.TimeInterval;

import com.dayorganiser.util.Interval;

public class TimelineAxis extends AbstractAxis implements Axis
{
	public TimelineAxis(String name)
	{
		super(name);
	}

	//___________________________________________________________________________

	public synchronized boolean zoom(int scale)
	{
		TimeInterval timeInterval = (TimeInterval) interval;
		TimeInterval visibleInterval = (TimeInterval) visible;

		//calculate changes
		long offset = ((visibleInterval.getFinishTime() - visibleInterval.getStartTime()) / 50) * scale;

		//die gracefully if we are trying to view outside the interval
		if ((visibleInterval.getStartTime() - offset) < timeInterval.getStartTime() &&
		   (visibleInterval.getFinishTime() + offset) > timeInterval.getFinishTime())
		   return false;

		// make changes
		this.visible = new DefaultTimeInterval(visibleInterval.getStartTime() - offset,
		   visibleInterval.getFinishTime() + offset);

		// notify changes
		fireZoomChanged();
		return true;
	}

	//___________________________________________________________________________

	public Object getValue(int width, int point)
	{
		TimeInterval visibleInterval = (TimeInterval) visible;
		float ppm = (float) width / (float) (visibleInterval.getFinishTime() - visibleInterval.getStartTime());
		float value = (point / ppm) + visibleInterval.getStartTime();
		return new Long(new Float(value).longValue());
	}

	public int getGranularity(int width, int strWidth)
	{
		int lastGranularity = TimelineConstants.GRANULARITY_YEAR;
		for (int i= TimelineConstants.GRANULARITY_SECOND; i <=  TimelineConstants.GRANULARITY_YEAR; i++)
		{
			if (getWidth(width,i) > strWidth)
		   {
				lastGranularity = i;
				return i;
		   }
		}
		return lastGranularity;
	}

	public int getPoint(int width, Object c)
	{
		long calendar = ((Long) c).longValue();
		float ppm = (float) width / (float) ((Long)visible.getRange()).longValue();
		float value = (calendar - ((TimeInterval)visible).getStartTime()) * ppm;
		return new Float(value).intValue();
	}

	public double getWidth(int width, int granularity)
	{
		float ppm = (float) width / (float) ((Long)visible.getRange()).longValue();

		TimeInterval visibleInterval = (TimeInterval) visible;

		// add one to the granularity to calculate the first offset
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date(visibleInterval.getStartTime()));
		cal.add(getField(granularity),1);

		// the pixel conversion
		float value = (cal.getTime().getTime() - visibleInterval.getStartTime()) * ppm;
		return new Float(value).intValue();
	}

	public Iterator visibleIterator(int granularity)
	{
		return new GranularityIterator((TimeInterval) visible, getField(granularity));
	}

	//_Listeners_________________________________________________________________

	/** adds a new timeline listener*/
	public void addAxisListener(AxisListener listener)
	{
		synchronized (listeners)
		{
		   listeners.add(listener);
		}
	}

	/** removes a timeline listener*/
	public void removeAxisListener(AxisListener listener)
	{
		synchronized (listeners)
		{
		   listeners.remove(listeners);
		}
	}

	//___________________________________________________________________________

	protected void fireZoomChanged()
	{
		synchronized (listeners)
		{
		   for (Iterator iter = listeners.iterator(); iter.hasNext();)
			   ((AxisListener) iter.next()).granularityChanged();
		}
	}

	//___________________________________________________________________________
	/** returns the Calendar equiv field for this granularity
		this should be in a helper method*/

	protected static int getField(int granularity)
	{
		int value = Calendar.SECOND;
		switch (granularity)
		{
			case TimelineConstants.GRANULARITY_YEAR:
				value = Calendar.YEAR;
				break;
		   case TimelineConstants.GRANULARITY_MONTH:
				value = Calendar.MONTH;
				break;
		   case TimelineConstants.GRANULARITY_DAY:
				value = Calendar.DAY_OF_WEEK;
				break;
		   case TimelineConstants.GRANULARITY_HOUR:
			   value = Calendar.HOUR;
				break;
		   case TimelineConstants.GRANULARITY_MINUTE:
				value = Calendar.MINUTE;
				break;
		   case TimelineConstants.GRANULARITY_SECOND:
				value = Calendar.SECOND;
	  		   break;
		}
		return value;
	}

	protected class GranularityIterator implements Iterator
	{
		protected int field;
		protected TimeInterval interval;

		protected Calendar position;

		public GranularityIterator(TimeInterval interval, int field)
		{
		   this.field = field;
		   this.interval = interval;
		   this.position = Calendar.getInstance();
		   position.setTime(new Date(interval.getStartTime()));
		   for (int i=field+1; i < 14; i++)
				position.clear(i);
		   //next();
		}

		public boolean hasNext()
		{
		   return position.getTime().getTime() < interval.getFinishTime();
		}

		public Object next()
		{
		   // get the value
		   long value = position.getTime().getTime();

		   // move on to the next value in the granularity
		   position.add(field,1);
		   return new Long(value);
		}

		public void remove()
		{
		}
	}
}