package com.dayorganiser.jgantt.model.drawing;

import com.dayorganiser.jgantt.drawing.drawing.DrawingConstants;

import com.dayorganiser.util.Interval;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
  * The vector model is the default implementation of the DrawingModel
  * and is designed to be used by as a default implementation in most
  * situation */
public class VectorModel extends AbstractModel
{
	protected Object axises [] = new Object[2];

	protected Collection entries = new ArrayList();

	protected Map axisIntervals [] = new Map[2];

	protected Map attributes = new HashMap(); // EntryKey --> Attributes
	protected Map plotters = new HashMap();
	protected Map proxies = new HashMap();

	public VectorModel(Object xAxis, Object yAxis)
	{
		axises[DrawingConstants.HORIZONTAL_AXIS] = xAxis;
		axises[DrawingConstants.VERTICAL_AXIS] = yAxis;
	}

	//___________________________________________________________________________

	public void addEntry(Object key, Interval hozInterval, Interval vertInterval,
		Object plotter, Object proxy, Map attribs)
	{
		entries.add(key);

		addEntryInterval(key, hozInterval, DrawingConstants.HORIZONTAL_AXIS);
		addEntryInterval(key, vertInterval, DrawingConstants.VERTICAL_AXIS);

		// register the plotter & proxy
		plotters.put(key, plotter);
		proxies.put(key, proxy);

		if (attribs != null)
			attributes.put(key, attribs);
	}

	//___________________________________________________________________________

	public Map getAttributes(Object key)
	{
		Map attributes = (Map) this.attributes.get(key);
		return attributes != null ? attributes : Collections.EMPTY_MAP;
	}

	//___________________________________________________________________________

	public Object getAxis(Object o, int orientation)
	{
		return axises[orientation];
	}

   public Collection getEntries()
	{
		return entries.size() > 0 ? entries : Collections.EMPTY_SET;
	}

	public Interval getInterval(Object o, int orientation)
	{
		Map map = axisIntervals[orientation];
		return map != null ? (Interval) map.get(o) : null;
	}

	//___________________________________________________________________________

	public Object getPlotter(Object key)
	{
		return plotters.get(key);
	}

	public Object getProxy(Object key)
	{
		return proxies.get(key);
	}

	//___________________________________________________________________________

	/** register the interval */
	protected void addEntryInterval(Object key, Interval interval, int orientation)
	{
		Map map = axisIntervals[orientation];
		if (map == null)
		{
		   map = new HashMap();
			axisIntervals[orientation] = map;
		}
		map.put(key, interval);
	}
}