package com.dayorganiser.jgantt.model.range;

import java.util.Date;

import com.dayorganiser.jgantt.model.Axis;

import com.dayorganiser.jgantt.model.axis.AxisListener;

import com.dayorganiser.util.Interval;
import com.dayorganiser.util.Trace;

import com.dayorganiser.util.time.DefaultTimeInterval;

/**
	The RangeModel is the model commonly used by the JScrollBar class
	this interface therefore allows JScrollBar to update the time on the timeline
	creating a very loose link between the timeline renderer and the scroll bar

	this can be converted to be none time specific
*/
public class TimelineRangeModel extends AbstractRangeModel
{
	private static String THIS = TimelineRangeModel.class.getName();
	private static boolean TRACE = Trace.getEnable(THIS);

	public static float TIMERANGE = 100 - 0; //has to be a float for division purposes

	protected Axis axis;

	public TimelineRangeModel(Axis axis)
	{
		this.axis = axis;
		axis.addAxisListener(new LocalAxisListener());
	}

	//___________________________________________________________________________

	public int getValue()
	{
		Interval interval = axis.getInterval();
		long offset = ((Long) interval.getRange()).longValue();

		Interval visible = axis.getVisible();
		float ppu = TIMERANGE / offset;

		long range = ((Long) visible.getRange()).longValue() /2 ;
		range = range + ( ((Long)visible.getStart()).longValue() -
		   ((Long)interval.getStart()).longValue());
		float testing = (range * ppu);
		int returnValue = new Float(testing).intValue();
		return returnValue;
    }

	/** the extent is the visible portion on the screen */
	public int getExtent()
	{
		Interval interval = axis.getInterval();
		long offset = ((Long) interval.getRange()).longValue();

		Interval visible = axis.getVisible();
		float ppu = TIMERANGE / offset;

		long range = ((Long)visible.getRange()).longValue();

		float testing = (range * ppu);
		return new Float(testing).intValue();
	}

	//___________________________________________________________________________

   public int getMinimum()
	{
		return 0;
	}

	/** get maximum is buggy */
	public int getMaximum()
	{
		return (int) TIMERANGE;
	}

	//___________________________________________________________________________

	public void setValue(int testing)
	{
		if (TRACE) Trace.out.println(THIS + ".setValue() init " + testing);

		Interval interval = axis.getInterval();
		long offset = ((Long)interval.getRange()).longValue();
		float ppu   = TIMERANGE / offset;

		long out = new Float((testing) / ppu).longValue();
		long location = ((Long)interval.getStart()).longValue() + out;

		if (TRACE)
		{
		   Trace.out.println(THIS + ".setValue() " + "start:" + interval.getStart());
		   Trace.out.println(THIS + ".setValue() " + "finish:" + interval.getFinish());
		   Trace.out.println(THIS + ".setValue() " + "location:" + location);
		}

		Interval visible = axis.getVisible();
		long range = ((Long)visible.getRange()).longValue();

	   if (TRACE) Trace.out.println(THIS + ".setValue() " + "vf - vs = " + range);

		range = range / 2;

		if (TRACE)
		{
		   Trace.out.println(THIS + ".setValue() " +" range (vf -vs) / 2   " + range);
		   Trace.out.println(THIS + ".setValue() " + "calc ((vf - vs) /2) + is  " +
				(range + ((Long)interval.getStart()).longValue()));
		}

		// take the largest of the two values
		if (location - range < ((Long)interval.getStart()).longValue())
		{
		   if (TRACE) Trace.out.println(THIS + ".setValue() " + "using lower bound");
		   location = ((Long)interval.getStart()).longValue() + range;
		}

		if (location + range > ((Long)interval.getFinish()).longValue())
		{
		   if (TRACE) Trace.out.println(THIS + ".setValue() " + "using upper bound");
		   location = ((Long)interval.getFinish()).longValue() - range;
		}

		Date newStart = new Date(location - range);
		int granularity = axis.getGranularity(500,2); // HACK!!
		Date newFinish = new Date(location + range);

	   if (TRACE)
		{
		   Trace.out.println(THIS + ".setValue() " + "fixed lo " + location);
		   Trace.out.println(THIS + ".setValue() " + "fixed lo " + new Date(location));
		}

		axis.setVisible(new DefaultTimeInterval(
			newStart.getTime(),
		   newFinish.getTime()));
   }

	//___________________________________________________________________________

	protected class LocalAxisListener implements AxisListener
	{
		public void intervalChanged()
		{
		   fireStateChanged();
		}

		public void granularityChanged()
		{
		   fireStateChanged();
		}
	}
}