package com.dayorganiser.treetable.component.swing;

import com.dayorganiser.treetable.drawing.editor.TreeTableCellEditor;

import com.dayorganiser.treetable.drawing.renderer.TreeTableCellRenderer;

import com.dayorganiser.treetable.model.TreeTableModelAdapter;

import java.awt.Dimension;
import java.awt.Component;
import java.awt.Graphics;

import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;

import javax.swing.table.TableModel;

import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeSelectionModel;

import javax.swing.AbstractCellEditor;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import javax.swing.tree.TreeModel;

/** this is a wrapper around JTable required inorder to synchronize the two models */

public class JTreeTable extends JTable
{
	protected JTree tree;

	public JTreeTable(TableModel tableModel, TreeModel treeModel)
	{
		// Create the tree. It will be used as a renderer and editor.
		this.tree = createTree(treeModel);

		// configuration options for the table
		setIntercellSpacing(new Dimension(0, 0));
	   setShowGrid(false);

		// Install the tree editor renderer and editor
	   setDefaultRenderer(TreeModel.class, new TreeTableCellRenderer(tree));
		setDefaultEditor(TreeModel.class, new TreeTableCellEditor(tree));

		// Install a tableModel representing the visible rows in the tree.
	   setModel(new TreeTableModelAdapter( tableModel, tree));

		LocalTreeSelectionModel selectionModel = new LocalTreeSelectionModel();

		// tree & table have the same model
		tree.setSelectionModel(selectionModel);
		setSelectionModel(selectionModel.getSelectionModel());
   }

	//___________________________________________________________________________

	/** Workaround for BasicTableUI abnomaly. Make sure the UI never tries to
	 * paint the editor. The UI currently uses different techniques to
	 * paint the renderers and editors and overriding setBounds() below
	 * is not the right thing to do for an editor. Returning -1 for the
	 * editing row in this case, ensures the editor is never painted.
	 **/

	public int getEditingRow()
	{
		return getColumnClass(editingColumn) == TreeModel.class ? -1 : editingRow;
	}

	//___________________________________________________________________________

	public JTree getTree()
	{
		return tree;
	}

	//___________________________________________________________________________

	protected JTree createTree(TreeModel model)
	{
		JTree tree = new JTree(model);
		tree.setRootVisible(false);
		tree.setRowHeight(getRowHeight()); // Make the tree and table row heights the same
		tree.setShowsRootHandles(true);
		return tree;
	}

	//___________________________________________________________________________

	protected class LocalTreeSelectionModel extends DefaultTreeSelectionModel
	{
		public LocalTreeSelectionModel()
		{
		}

		/** access method to the list model previously unavaialble in the DefaultTreeModel why?*/
		public ListSelectionModel getSelectionModel()
		{
		   return listSelectionModel;
		}
	}
}