package com.dayorganiser.treetable.drawing.renderer;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Rectangle;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JPanel;
import javax.swing.JTree;

import javax.swing.SwingUtilities;

import javax.swing.tree.TreePath;

import javax.swing.plaf.basic.BasicTreeUI;

import javax.swing.table.TableCellRenderer;

import javax.swing.tree.TreeModel;

/** This is a tree renderer that fits into a table cell */
public class TreeTableCellRenderer extends JComponent implements TableCellRenderer
{
	protected MouseListener localListener;
	protected JTree tree;
	protected JTable table;

	protected int visibleRow = 0;

	public TreeTableCellRenderer(JTree tree)
	{
		//this.add(tree);
		this.tree = tree;
	}

	//___________________________________________________________________________

	/** this should only be called once*/
	public Component getTableCellRendererComponent(JTable table, Object value,
		boolean isSelected, boolean hasFocus, int visibleRow, int column)
	{
		this.table = table;
		this.visibleRow = visibleRow;
		if (localListener == null)
		{
		   this.localListener = new LocalListener();
			table.addMouseListener(localListener);
		}
		tree.setBackground(isSelected ? table.getSelectionBackground() : table.getBackground());
		return this;
	}

	//___________________________________________________________________________

	int rowHeight=20;
   public void setBounds(int x, int y, int w, int h)
	{
		tree.setBounds(x, 0, w, h); //JTreeTable.getHeight?
		if (h == 0 || h == rowHeight)
		   return;

	   rowHeight = h;
		System.out.println("row height = " + rowHeight);
		tree.setRowHeight(rowHeight);
	}

	//___________________________________________________________________________

	public void paint(Graphics g)
	{
		g.translate(0, -visibleRow * tree.getRowHeight());
		tree.paint(g);
	}

	//___________________________________________________________________________

	protected class LocalListener implements MouseListener
	{
		public void mouseClicked(MouseEvent e)
		{
	      if (tree == null || !tree.isEnabled())
				return;

   	   if (!SwingUtilities.isLeftMouseButton(e))
				return;

			TreePath  path = tree.getPathForLocation(Math.max(e.getX(), 30), e.getY());
			if(path == null)
				return;

		   if (SwingUtilities.isLeftMouseButton(e))
		   {
				if (tree.isExpanded(path))
				   tree.collapsePath(path);
				else
				   tree.expandPath(path);

				tree.setSelectionPath(path);
		   }
		}

		//________________________________________________________________________

		public void mousePressed(MouseEvent e)
		{
		}

		public void mouseReleased(MouseEvent e)
		{
		}

		//________________________________________________________________________

		public void mouseEntered(MouseEvent e)
		{
		}

		public void mouseExited(MouseEvent e)
		{
		}
	}
}