package com.dayorganiser.treetable.model;

import java.util.Iterator;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import javax.swing.JTree;

import javax.swing.table.TableModel;

import javax.swing.tree.TreePath;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;

import javax.swing.table.TableModel;

import javax.swing.tree.TreeModel;

public class TreeTableModelAdapter implements TableModel
{
	protected JTree tree;
   protected TableModel model;
	protected Vector listeners = new Vector();

	public TreeTableModelAdapter(TableModel model, JTree tree)
	{
		this.model = model;
		this.tree = tree;

		tree.addTreeExpansionListener(new LocalExpansionListener());
	}

	//___________________________________________________________________________

	public void addTableModelListener(TableModelListener listener)
	{
		model.addTableModelListener(listener);
		listeners.add(listener);
	}

	public void removeTableModelListener(TableModelListener listener)
	{
		model.removeTableModelListener(listener);
		listeners.remove(listener);
	}

	//___________________________________________________________________________

	public void fireTableDataChanged()
	{
		TableModelEvent e = new TableModelEvent(this,0, getRowCount());
		for (Iterator iter = listeners.iterator(); iter.hasNext();)
			((TableModelListener)iter.next()).tableChanged(e);
	}

	//_TableModel interface______________________________________________________

	public int getColumnCount()
	{
	   return model.getColumnCount();
   }

   public int getRowCount()
	{
	   return model.getRowCount();
   }

	//___________________________________________________________________________

	public String getColumnName(int column)
	{
		return model.getColumnName(column);
	}

   public Class getColumnClass(int column)
	{
	   return model.getColumnClass(column);
   }

	//___________________________________________________________________________

   public Object getValueAt(int row, int column)
	{
		return model.getValueAt(row, column);
   }

	public void setValueAt(Object value, int row, int column)
	{
		model.setValueAt(value, row, column);
   }

	//___________________________________________________________________________

   public boolean isCellEditable(int row, int column)
	{
		return model.isCellEditable(row, column);
	}

	//___________________________________________________________________________

	protected class LocalExpansionListener implements TreeExpansionListener
	{
		/** Don't use fireTableRowsInserted() here; the selection model would get  updated twice*/
		public void treeExpanded(TreeExpansionEvent event)
		{
		   fireTableDataChanged();
		}

		public void treeCollapsed(TreeExpansionEvent event)
		{
		   fireTableDataChanged();
		}
	}
}