package com.dayorganiser.util.files;

import java.io.File;
import java.io.InputStream;

import java.net.URL;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;
import java.util.WeakHashMap;

import com.dayorganiser.util.Trace;

/* This is a utility class used to find the CLASSDATA directory for this project,
	from the CLASSPATH one example is for the Zoom icons used by the test2 application,
	see the CLASSDATA_INFO string below for more information. */

public class Hive
{
	/** as the hive may or may not be initialised it is safest to just set this flag
		in the code if you need to debug what is going wrong */
	private static String THIS = Hive.class.getName();
	private static boolean TRACE = false;

	/** url prefix */
	public static String FILE_PREFIX = "file:///";

	/* this is the file that we are looking for by iterating through
		the ClassPath directories.  This file will be located in the root
      directory relative to other important items, like images.  When we
      find this file, we find the root for the project really. */
	public static String CLASSDATA_INFO = "histogram.info";

	/**
		this is the file directory that we are looking for when it comes to tracing
		or user dependent configuration */
	public static String CONFIG_INFO =  "config.info";

	/** this the order of precedence of the keys of which to retrieve the file */
	private static Vector lookupOrder = new Vector();

	private static Map paths = new WeakHashMap();
	static
	{
		if (!registerPath(CLASSDATA_INFO)) //classdata will be used last
		   System.out.println("WARNING: hive failed to initialised ClassData");
		registerPath(CONFIG_INFO);
	}

	//___________________________________________________________________________

	public static boolean registerPath(String pathFile)
	{
		String path = getLocationFromClassPath(pathFile);
		if (path == null)
		   return false;

	   paths.put(pathFile, path);
		lookupOrder.add(0,pathFile); //this is the preferred order
		return true;
	}

	public boolean unregisterPath(String pathFile)
	{
		return lookupOrder.remove(pathFile);
	}
	//___________________________________________________________________________

	/* converts a string to a URL, this will not throw exceptions back,
		it will catch them, print them to error and return NULL in case of a failure */
	public static URL getURL(String path)
	{
		for (Iterator iter = lookupOrder.iterator(); iter.hasNext();)
		{
		   String classPath = (String) paths.get((String) iter.next());
			try
	   	{
		   	URL url = new URL("file:///" + classPath + "/" + path);
				InputStream is = url.openStream();
				is.close();
				return url;
			}
	   	catch (java.net.MalformedURLException me)
		   {
				if (TRACE) Trace.out.println(THIS + ".getURL() " + " invalid URL file:"
				   + classPath +"/" + path);
			}
		   catch (java.io.IOException io)
		   {
				if (TRACE) Trace.out.println(THIS + ".getURL() " + " could not open stream:"
				   + classPath +"/" + path);
		   }
		}
		return null;
	}

	//___________________________________________________________________________

	/* gets a complete listing of the ClassPath.  This is used to build the URL which
		will be returned by getURL */
	private static String getLocationFromClassPath(String filename)
	{
		String classPath = System.getProperty("java.class.path");
		int i = 0;

		while (i >= 0)
		{
		   // position of the seperator character in the classpath
			int j = classPath.indexOf(File.pathSeparatorChar, i);
			String item;
			if (j < 0)
			{
				item = classPath.substring(i);
				i = j;
			}
			else
			{
				item = classPath.substring(i, j);
				i = j+1;
			}

		   // we need to add debug statements
			if (TRACE) Trace.out.println(item);

		   // attempt to locate a file that matches the item
			File f = new File(item);
			if (f.isDirectory())
			{
            // the classpath specifies a directory, try to open the file
				// CLASSDATA_INFO, relative to the directory
				if (new File(f, filename).isFile())
				{
				   if (TRACE) Trace.out.println(THIS + ".getLocationFromClassPath() " +
						"found file: " + item + "/" +CLASSDATA_INFO);
				   return item; // File found success
				}
			}
		}
		return null; // we did not find the CLASSDATA_INFO file, give up
	}
}