package test.jgantt.demo;

import java.awt.Container;

import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JTable;
import javax.swing.JToolBar;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import com.dayorganiser.jgantt.model.Axis;

import com.dayorganiser.jgantt.model.axis.FloatAxis;

import com.dayorganiser.jgantt.drawing.Drawing;

import com.dayorganiser.jgantt.drawing.drawing.DrawingConstants;

import com.dayorganiser.jgantt.drawing.plotter.axis.TimelinePlotter;

import com.dayorganiser.jgantt.drawing.plotter.entry.EntryConstants;
import com.dayorganiser.jgantt.drawing.plotter.entry.DefaultEntry;
import com.dayorganiser.jgantt.drawing.plotter.entry.TaskBarEntry;

import com.dayorganiser.jgantt.drawing.plotter.marker.MarkerConstants;
import com.dayorganiser.jgantt.drawing.plotter.marker.IconMarker;

import com.dayorganiser.jgantt.drawing.proxy.entry.DefaultEntryProxy;

import com.dayorganiser.jgantt.drawing.scheme.CombinedSchemeProvider;
import com.dayorganiser.jgantt.drawing.scheme.SchemeProvider;

import com.dayorganiser.jgantt.drawing.scheme.color.ColorSchemeProvider;

import com.dayorganiser.jgantt.drawing.scheme.font.FontProvider;

import com.dayorganiser.jgantt.model.DrawingModel;

import com.dayorganiser.jgantt.model.axis.TimelineAxis;

import com.dayorganiser.jgantt.model.axis.actions.ZoomAction;

import com.dayorganiser.jgantt.drawing.component.DrawingComponent;

import com.dayorganiser.jgantt.panel.diagram.actions.FitToAllAction;

import com.dayorganiser.util.FloatInterval;
import com.dayorganiser.util.Trace;
import com.dayorganiser.util.Interval;

import com.dayorganiser.util.files.Hive;

import com.dayorganiser.util.time.DefaultTimeInterval;

public abstract class GanttDemo extends AbstractDemo
{
	public static String XAXIS_KEY = "xAxis";
	public static String YAXIS_KEY = "yAxis";

	public static String DEFAULT_ENTRY_PLOTTER = "DefaultEntryPlotter";
	public static String DEFAULT_ENTRY_PROXY = "DefaultEntryProxy";

	public static String MARKER_PLOTTER = "DefaultMarker";
	public static String MARKER_PROXY = "MarkerProxy";

	public static String SHIFT_ENTRY_PLOTTER = "ShiftEntryPlotter";
	public static String SHIFT_ENTRY_PROXY = "ShiftEntryProxy";

	public static String TIMELINE_PLOTTER = "TimelinePlotter";
	public static String TIMELINE_PROXY = "TimelinePlotter";

	protected Axis xAxis, yAxis;
	protected TableModel model;

	public GanttDemo(Container container)
	{
		super (container);
	}

	//___________________________________________________________________________

	protected Axis createAxis(int position)
	{
		return position == DrawingConstants.VERTICAL_AXIS
		   ? (Axis) new FloatAxis(YAXIS_KEY)
		   : (Axis) new TimelineAxis(XAXIS_KEY);
	}

	//___________________________________________________________________________

	protected Drawing createDrawing()
	{
		this.xAxis = createAxis(DrawingConstants.HORIZONTAL_AXIS);
		this.yAxis = createAxis(DrawingConstants.VERTICAL_AXIS);

		Drawing drawing = createDrawingComponent();

		drawing.addAxis("xAxis", xAxis);
		drawing.addAxis("yAxis", yAxis);
		return drawing;
	}

	protected Drawing createDrawingComponent()
	{
		DrawingComponent drawing = new DrawingComponent();
		drawing.addPlotter(DEFAULT_ENTRY_PLOTTER, new DefaultEntry());
		// drawing.addPlotter(DEFAULT_ENTRY_PLOTTER, new SVGEntry(Hive.getURL("svg/textRotate.svg")));
		drawing.addPlotter(TIMELINE_PLOTTER,
			new TimelinePlotter(xAxis,true));
		drawing.addPlotter(SHIFT_ENTRY_PLOTTER, new TaskBarEntry());
		drawing.addPlotter(MARKER_PLOTTER, new IconMarker(
		   new ImageIcon(Hive.getURL("images/milestone.gif"),"a tick").getImage(), 20));

		SchemeProvider provider = createSchemeProvider();

		HashMap attributes = new HashMap();
		attributes.put(EntryConstants.BACKGROUND_SCHEME, "shift");
		drawing.addProxy(SHIFT_ENTRY_PROXY, new DefaultEntryProxy(0, provider, attributes));

		attributes = new HashMap();
		attributes.put(EntryConstants.BACKGROUND_SCHEME, "test");
		drawing.addProxy(DEFAULT_ENTRY_PROXY, new DefaultEntryProxy(0, provider, attributes));
		drawing.addProxy(TIMELINE_PROXY, new DefaultEntryProxy(0, provider, attributes));

		attributes = new HashMap();
		attributes.put(MarkerConstants.BACKGROUND_SCHEME, "yellowblue");
		attributes.put(MarkerConstants.FONT_SCHEME, "markerFont");
		drawing.addProxy(MARKER_PROXY, new DefaultEntryProxy(0, provider, attributes));

		return drawing;
	}

	//___________________________________________________________________________

	protected SchemeProvider createSchemeProvider()
	{	// create the color-scheme provider
		SchemeProvider colorProvider = new ColorSchemeProvider(
		   Hive.getURL("drawing/color/colorscheme.properties"));

		SchemeProvider fontProvider = new FontProvider(
		   Hive.getURL("drawing/font/fontscheme.properties"));

		// add them to the combined provider
		CombinedSchemeProvider provider = new CombinedSchemeProvider();
		provider.add(colorProvider);
		provider.add(fontProvider);
		return provider;
	}

	//___________________________________________________________________________

	protected JTable createTable()
	{
		JTable table = new JTable();
		table.setRowHeight(18);
		return table;
	}

	//___________________________________________________________________________

	protected JToolBar createToolBar()
	{
		JToolBar toolbar = super.createToolBar();
		Axis axis = null;
		toolbar.add(new ZoomAction("ZOOM IN",  new ImageIcon(Hive.getURL("images/zoomIn.gif"),"ZOOM IN"),  -10, axis));
   	toolbar.add(new ZoomAction("ZOOM OUT", new ImageIcon(Hive.getURL("images/zoomOut.gif"),"ZOOM OUT"), +10, axis));
	   toolbar.addSeparator();

		// toolbar.add(new FitToAllAction("FIT VISIBLE", new ImageIcon(Hive.getURL("images/zoomAll.gif"), "FIT VISIBLE"), true, model, axis));
   	// toolbar.add(new FitToAllAction("FIT ALL", new ImageIcon(Hive.getURL("images/zoomAll.gif"), "FIT ALL"), false, model, axis));
	   toolbar.addSeparator();

		return toolbar;
	}
}