package test.jgantt.demo;

import com.dayorganiser.jgantt.drawing.Drawing;
import com.dayorganiser.jgantt.drawing.drawing.DrawingConstants;

import com.dayorganiser.jgantt.model.Axis;
import com.dayorganiser.jgantt.model.DrawingModel;
import com.dayorganiser.jgantt.model.ModelConstants;

import com.dayorganiser.jgantt.model.drawing.VectorModel;

import com.dayorganiser.jgantt.combined.swing.table.DrawingCellRenderer;

import com.dayorganiser.util.FloatInterval;
import com.dayorganiser.util.Interval;
import com.dayorganiser.util.Trace;

import com.dayorganiser.util.time.DefaultTimeInterval;
import com.dayorganiser.util.time.TimeInterval;

import test.jgantt.demo.project.Task;

import java.awt.Container;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.Enumeration;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;

import java.util.Collections;

import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.WindowConstants;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumn;

public class Test2 extends GanttDemo
{
	private static String THIS = Test2.class.getName();
	private static boolean TRACE = Trace.getEnable(THIS);

	public static int SHIFT_TYPE = 1;

	protected Interval timeInterval;

	public Test2(Container container)
	{
		super(container);
	}

	//___________________________________________________________________________

	public static void main(String args[])
	{
		if (!TRACE) System.out.println(THIS + ".main() initializing example");

		// fit up the size and display it
		JFrame frame = new JFrame("Test2");
		frame.setSize(500,500);

		// initialise the gantt chart
		Test2 test = new Test2(frame.getContentPane());
		test.initialise();

		// display the gantt chart
		frame.pack();
		frame.setVisible(true);

	   // dispose of the frame when hit the close button on the frame
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		frame.addWindowListener
		(
		   new WindowAdapter ()
		   {
				public void windowClosed (WindowEvent e)
				{
		         System.exit(0);
		      }
		   });
	}


	//___________________________________________________________________________

	protected void initialiseTable(JTable table)
	{
		Drawing drawing = createDrawing();

		for (Enumeration enum = table.getColumnModel().getColumns(); enum.hasMoreElements();)
		{
		   TableColumn tableColumn = 	(TableColumn) enum.nextElement();

		   if (!(tableColumn.getHeaderValue().equals(DrawingModel.class.toString())))
				continue;

			tableColumn.setHeaderRenderer(new DrawingCellRenderer(drawing));
	   	tableColumn.setCellRenderer(new DrawingCellRenderer(drawing));

	      VectorModel diagramModel = new VectorModel("xAxis", "yAxis");

	   	diagramModel.addEntry(new Object(), xAxis.getInterval(), new FloatInterval(0,4),
		   	TIMELINE_PLOTTER, TIMELINE_PROXY, null);
			tableColumn.setHeaderValue(diagramModel);
		}
	}

	//___________________________________________________________________________

	protected Axis createAxis(int position)
  {
		Axis axis =	super.createAxis(position);

		Interval interval = position == DrawingConstants.VERTICAL_AXIS
		   ? (Interval) new FloatInterval(0, 4)
		   : (Interval) timeInterval;
		axis.setInterval(interval);
		return axis;
	}

	//___________________________________________________________________________

	protected Map createEntries(Connection connection)
	{
		Map results = null;
		try
		{
			Statement db_statement = connection.createStatement();
	   	ResultSet result = db_statement.executeQuery	("select * from MSP_TASKS order by TASK_OUTLINE_NUM");

			results = new HashMap(result.getFetchSize());
	   	while (result.next())
			{
			   Task task = new Task(result);
		      if (task.isValid())
		   	   results.put(new Integer(task.getUniqueID()), task);
	   	}
		}
		catch (Exception e)
		{
		}
		return results != null ? results : Collections.EMPTY_MAP;
	}

	//___________________________________________________________________________

	protected JTable createTable()
	{
		JTable table = super.createTable();

		Connection db_connection = getConnection( "jdbc:odbc:" + "msproject", "sa", "sql");
		Map entries = createEntries(db_connection); // create the entries i.e. Integer (ID) --> MSPTask
		this.timeInterval = getInterval(entries);

		DefaultTableModel model = (DefaultTableModel) createTableModel();
		table.setModel(model);

		initialiseTable(table);
		Drawing drawing = createDrawing();

		for (int i=0; i < entries.size(); i++)
		{
		   Integer key = new Integer(i+1); //entries are keyed from 0
		   Task task = (Task) entries.get(key);

		   Object values [] = getValues(task);
		   model.addRow(values); // insert a row into the model
		}
		return table;
	}

	//___________________________________________________________________________

	protected TableModel createTableModel()
	{
		DefaultTableModel model = new DefaultTableModel();

		Object cols [] = getColumns();

		for (int i=0; i < cols.length; i++)
			model.addColumn(cols[i]);
		return model;
	}

	//___________________________________________________________________________


	protected Object [] getColumns()
	{
		Object columns [] = new Object [3];
		columns [0] = "Name";
		columns [1] = "Description";
		columns [2] = DrawingModel.class;
		return columns;
	}

	//___________________________________________________________________________

	protected Interval getInterval(Map entries)
	{
		long start = Long.MAX_VALUE;
		long finish = Long.MIN_VALUE;

		for (int i=0; i < entries.size(); i++)
		{
		   Integer key = new Integer(i+1); //entries are keyed from 0
		   Task task = (Task) entries.get(key);
		   TimeInterval timeInterval = (TimeInterval) task.getInterval();

		   start = Math.min(start, timeInterval.getStartTime());
		   finish = Math.max(finish, timeInterval.getFinishTime());
		}

		return new DefaultTimeInterval(start, finish);;
	}

	//___________________________________________________________________________

	protected Object [] getValues(Task task)
	{
	   Object values [] = new Object [3];
	   values[0] = task.getID();
	   values[1] = task.getName();

	   VectorModel diagramModel = new VectorModel("xAxis", "yAxis");
	   values[2] = diagramModel;

		TimeInterval interval = (TimeInterval) task.getInterval();
	   if (interval.getFinishTime() > interval.getStartTime())
	   {
			diagramModel.addEntry(task.getID(), interval, new FloatInterval(0,4),
				getPlotter(task), getProxy(task), null);
	   }
	   else
	   {
			Map attributes = new HashMap();
			attributes.put(ModelConstants.ENTRY_TEXT, task.getName());

		   // to display a text marker
		   diagramModel.addEntry(task.getID() + task.getName() ,
			   new DefaultTimeInterval(interval.getStartTime(), Long.MAX_VALUE),
				new FloatInterval(0, 4),
				MARKER_PLOTTER, MARKER_PROXY, attributes); //task.getName();
	   }
		return values;
	}
	//___________________________________________________________________________

	protected String getPlotter(Task task)
	{
		return task.getTaskType() == SHIFT_TYPE ? SHIFT_ENTRY_PLOTTER
		   : DEFAULT_ENTRY_PLOTTER;
	}

	protected String getProxy(Task task)
	{
		return task.getTaskType() == SHIFT_TYPE ? SHIFT_ENTRY_PROXY :
		   DEFAULT_ENTRY_PROXY;
	}

	//___________________________________________________________________________

	private Connection getConnection(String url, String username, String password)
	{
		try
		{	// Load Sun's jdbc-odbc driver
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver").newInstance();
   		// Attempt to load database driver
   		return DriverManager.getConnection (url, username, password);
		}
		catch (Exception e) // driver not found
		{
			System.err.println ("Unable to load database driver");
			System.err.println ("Details : " + e);
		}
	   return null;
	}
}