package test.treetable.filesystem;

import java.io.File;

import java.util.Date;
import java.util.Enumeration;

import javax.swing.JTree;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultMutableTreeNode;

import javax.swing.tree.TreePath;

import javax.swing.table.TableModel;

import javax.swing.event.TableModelListener;

import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;

public class FileSystemModel extends DefaultTreeModel implements TableModel, TreeModel
{
	protected static String [] columnNames = {"Name", "Size", "Type", "Modified"};
	protected static Class [] columnTypes = {TreeModel.class, Integer.class, String.class, Date.class};

	protected JTree tree; // used for calculate the position

	public FileSystemModel()
	{
		super(new DefaultMutableTreeNode(new File(File.separator)));

		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) super.getRoot();
		addNodes(rootNode,0);
	}

	public void initialise(JTree tree)
	{
		this.tree = tree;
	}

	//_TableModel interface___________________________________________________

	public int getColumnCount()
	{
		return columnNames.length;
   }

	public String getColumnName(int column)
	{
		return columnNames[column];
   }

	public Class getColumnClass(int column)
	{
		return columnTypes[column];
   }


	public int getRowCount()
	{
		return tree.getVisibleRowCount();
	}

	public Object getValueAt(int row, int col)
	{
		Object node = getNode(row);
		return node != null ? getValueAt(node, col) : null;
	}

		/** not implemented yet */
	public void setValueAt(Object o, int i, int z)
	{
	}

	public boolean isCellEditable(int row,  int b)
	{
		return true;
	}

	//_table_helpers___________________________________________________________________

   public Object getValueAt(Object node, int column)
	{
	   File file = getFile(node);
		switch(column)
	   {
			case 0: return file.getName();
			case 1: return file.isFile() ? new Integer((int)file.length()) : new Integer(0);
			case 2: return file.getName();
			case 3: return new Date(file.lastModified());
	   }
		return null; // on failure null
	}

	//___________________________________________________________________________

   public void addTableModelListener(TableModelListener l)
	{
	}

   public void removeTableModelListener(TableModelListener l)
	{
	}

	//___________________________________________________________________________

	public boolean addNodes(DefaultMutableTreeNode parent, int level)
	{
		if (level == 2)
	   	return false;

		File file = (File) parent.getUserObject();
		String [] files = file.list();
		if (files == null) // skip files with no children
			return false;

		String path = file.getPath(); // cache the path
		for(int pos = 0; pos < files.length; pos++)
		{
			DefaultMutableTreeNode child = new DefaultMutableTreeNode(
				new File(path, files[pos]));

			insertNodeInto(child , parent, pos); // insert at a reasonable pos

			addNodes(child, level +1); // add our children too
		}
		return true;
	}

	//___________________________________________________________________________

	protected Object getNode(int row)
	{
	   TreePath treePath = tree.getPathForRow(row);
	   return treePath != null ? treePath.getLastPathComponent() : null;
	}
	//___________________________________________________________________________

	protected File getFile(Object node)
	{
		DefaultMutableTreeNode fileNode = ((DefaultMutableTreeNode)node);
	   return (File) fileNode.getUserObject();
	}
}